@@ -62,6 +62,7 @@
 
 #include <cstdlib>
 #include <unistd.h>
+#include <sstream>
 
 using namespace std;
 using namespace Qt;
@@ -887,7 +888,7 @@ bool MainWindowGUI::saveProjectAs() {
 	saveAct->setEnabled(false);
 	setTitle(false);
 	updateMostRecentMenu();
-	return false;
+	return true;
 }
 
 
@@ -920,20 +921,26 @@ void MainWindowGUI::exportToVideo()
 	}
 	else {
 		bool isCanceled = false;
-		char tmp[PATH_MAX];
+		std::stringstream key;
+		std::string k;
 		VideoEncoder enc;
 
-		sprintf(tmp, "startEncoder%d", active);
-		Preference start(tmp, "");
+		key << "startEncoder" << active;
+		k = key.str();
+		Preference start(k.c_str(), "");
 		enc.setStartCommand(start.get());
 
-		sprintf(tmp, "stopEncoder%d", active);
-		Preference stop(tmp, "");
+		key.str("");
+		key << "stopEncoder" << active;
+		k = key.str();
+		Preference stop(k.c_str(), "");
 		enc.setStopCommand(stop.get());
 
-		sprintf(tmp, "outputFile%d", active);
-		Preference output(tmp);
-		if (output.get()) {
+		key.str("");
+		key << "outputFile" << active;
+		k = key.str();
+		Preference output(k.c_str());
+		if (output.get() && *output.get()) {
 			enc.setOutputFile(output.get());
 		} else {
 			QString file = QFileDialog::
@@ -948,7 +955,8 @@ void MainWindowGUI::exportToVideo()
 		}
 
 		if ( enc.isValid() && isCanceled == false ) {
-			DomainFacade::getFacade()->exportToVideo(&enc);
+			DomainFacade::getFacade()->exportToVideo(&enc,
+					frameView->getPlaybackSpeed());
 		}
 		else if (!isCanceled){
 			int ret = QMessageBox::warning(this,
@@ -1138,7 +1146,6 @@ void MainWindowGUI::activateMenuOptions() {
 void MainWindowGUI::createMostRecentMenu() {
 	mostRecentMenu->clear();
 	mostRecentMenu->setTitle(tr("Open &Recent"));
-	PreferencesTool* pref = PreferencesTool::get();
 	Preference first("mostRecent", "");
 	if (first.get() && access(first.get(), R_OK) == 0) {
 		mostRecentAct->setVisible(true);
