@@ -27,12 +27,45 @@
 #include <iostream>
 #include <cstring>
 
-using namespace std;
+enum WarnIfExitOne { noWarn, doWarn };
+
+bool callSystem(const char* task, const char* commandLine,
+		WarnIfExitOne warn = doWarn) {
+	if (!commandLine || strlen(commandLine) == 0) {
+		Logger::get().logDebug("No process defined for '%s'", task);
+		return true;
+	}
+	Logger::get().logDebug("Attempting task '%s'", task);
+	int r = system(commandLine);
+	if (!WIFEXITED(r)) {
+		Logger::get().logFatal("Could not start task '%s': %s", task,
+				commandLine);
+		return false;
+	}
+	int code = WEXITSTATUS(r);
+	if (code == 0) {
+		Logger::get().logDebug("Task '%s' returned code 0: %s", task,
+				commandLine);
+		return true;
+	}
+	if (code == 1) {
+		if (warn == doWarn)
+			Logger::get().logWarning("Task '%s' returned code 1: %s", task,
+					commandLine);
+		else
+			Logger::get().logDebug("Task '%s' returned code 1: %s", task,
+					commandLine);
+		return true;
+	}
+	Logger::get().logFatal(
+		"Task '%s' returned code %d: %s",
+				task, r, commandLine);
+	return false;
+}
 
 
-CommandLineGrabber::CommandLineGrabber(const char* filePath, bool isProcess)
-		: ImageGrabber(filePath, isProcess)
-{
+CommandLineGrabber::CommandLineGrabber(const char* path)
+		: ImageGrabber(path) {
 	isInitSuccess = false;
 	this->prePoll = "";
 	this->startProcess = "";
@@ -40,13 +73,15 @@ CommandLineGrabber::CommandLineGrabber(const char* filePath, bool isProcess)
 }
 
 
-bool CommandLineGrabber::setPrePollCommand(const char *command) 
-{
+CommandLineGrabber::~CommandLineGrabber() {
+};
+
+
+bool CommandLineGrabber::setPrePollCommand(const char *command)  {
 	// This happens if the user doesn't uses pre poll
 	if ( strcmp(command, "") == 0) {
 		return true;
 	}
-	
 	prePoll = parseCommand(command);
 	if (prePoll != "") {
 		return true;
@@ -55,13 +90,11 @@ bool CommandLineGrabber::setPrePollCommand(const char *command)
 }
 
 
-bool CommandLineGrabber::setStartCommand(const char *command)
-{
+bool CommandLineGrabber::setStartCommand(const char *command) {
 	// This happens if the user doesn't uses start command
 	if ( strcmp(command, "") == 0) {
 		return true;
 	}
-	
 	startProcess = parseCommand(command);
 	if (startProcess != "") {
 		return true;
@@ -70,43 +103,26 @@ bool CommandLineGrabber::setStartCommand(const char *command)
 }
 
 
-bool CommandLineGrabber::setStopCommand(const char *command)
-{
-	stopProcess	= parseCommand(command);
+bool CommandLineGrabber::setStopCommand(const char *command) {
+	stopProcess = parseCommand(command);
 	return true;
 }
 
 
-bool CommandLineGrabber::init()
-{
-	if (isProcess) {
-		if (startProcess != "") {
-			Logger::get().logDebug("Attemting to start process");
-			system(startProcess.c_str());
-			
-		}
-		else {
-			return false;
-		}
-	}
-	return true;
-}
-
-
-bool CommandLineGrabber::tearDown()
-{
-	Logger::get().logDebug("Attemting to shutt down process");
-	if (stopProcess != "") {
-		system( stopProcess.c_str() );
+bool CommandLineGrabber::init() {
+	if (!isGrabberProcess())
 		return true;
-	}
-	return false;
+	return callSystem("start grabber", startProcess.c_str());
 }
 
 
-bool CommandLineGrabber::grab()
-{
-	if ( prePoll.empty() || system(prePoll.c_str()) != 0 ) {
+bool CommandLineGrabber::tearDown() {
+	return callSystem("stop grabber", stopProcess.c_str());
+}
+
+
+bool CommandLineGrabber::grab() {
+	if ( !callSystem("grab", prePoll.c_str(), noWarn) ) {
 		isInitSuccess = false;
 		return false;
 	}
@@ -114,20 +130,22 @@ bool CommandLineGrabber::grab()
 }
 
 
-string CommandLineGrabber::parseCommand(const char * command)
-{
-	string tmp = command;
-	int spaceIdx = tmp.find(" ", 0);
+std::string CommandLineGrabber::parseCommand(const char * command) {
+	std::string tmp = command;
+	int spaceIdx = Util::endOfArgument(command) - command;
 	std::string commandName = tmp.substr(0, spaceIdx);
 	std::string path;
 	if (Util::checkCommand(&path, commandName.c_str())) {
 		tmp.replace(0, spaceIdx, path);
 		int index = tmp.find("$IMAGEFILE");
 		if (index != -1) {
-			tmp.replace(index, strlen("$IMAGEFILE"), string(filePath));
+			tmp.replace(index, (int) strlen("$IMAGEFILE"), string(filePath()));
 		}
 		return tmp;
 	}
 	return "";
 }
 
+bool CommandLineGrabber::isGrabberProcess() {
+	return !startProcess.empty();
+}
