@@ -18,14 +18,14 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 #include "src/technical/util.h"
-
-#include "src/technical/libng/grab-ng.h"
+#include "src/foundation/logger.h"
 
 #include <ext/stdio_filebuf.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/ioctl.h>
 #include <unistd.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -33,6 +33,10 @@
 #include <cassert>
 #include <istream>
 #include <ftw.h>
+#include <glob.h>
+#include <linux/videodev2.h>
+#include <sstream>
+#include <new>
 
 using namespace std;
 
@@ -86,20 +90,27 @@ int removeFileOrDirectory(const char *path, const struct stat *,
 		int flag, struct FTW *info) {
 	switch (flag) {
 	case FTW_D:
-		if (info->level != info->base) {
-			if (0 != rmdir(path))
+	case FTW_DP:
+		if (info->level != 0) {
+			if (0 != rmdir(path)) {
+				Logger::get().logWarning("Could not remove directory %s", path);
 				return FTW_STOP;
+			}
 		}
 		break;
 	default:
-		if (0 != unlink(path))
+		if (0 != unlink(path)) {
+			Logger::get().logWarning("Could not remove file %s", path);
 			return FTW_STOP;
+		}
 		break;
 	}
 	return FTW_CONTINUE;
 }
 
-const char* endOfArgument(const char* in) {
+}
+
+const char* Util::endOfArgument(const char* in) {
 	enum CharClass {
 		normalChar,
 		backslashChar,
@@ -124,21 +135,21 @@ const char* endOfArgument(const char* in) {
 			{ dquote, dquote, dquote, dquote, dquote } // dqbackslash
 	};
 	char c = *in;
-	while (c != '\0' && state != end) {
+	while (c != '\0') {
 		CharClass cc = c == '\\'? backslashChar
 				: c == '\''? squoteChar
 						: c == '"'? dquoteChar
-								: c == ' '? space
+								: c == ' ' || c == '\t'? space
 										: normalChar;
 		state = transition[state][cc];
+		if (state == end)
+			return in;
 		++in;
 		c = *in;
 	}
 	return in;
 }
 
-}
-
 bool Util::checkCommand(std::string* pathOut, const char* command) {
 	assert(command != 0);
 	std::string which("which ");
@@ -158,70 +169,59 @@ bool Util::checkCommand(std::string* pathOut, const char* command) {
 	bool bad = bufStream.bad();
 	int status = pclose(fp);
 	int exitStatus = WEXITSTATUS(status);
-	return !bad && exitStatus < 2;
+	return !bad && exitStatus == 0;
 }
 
+namespace {
+
+template <std::size_t S> void setString(std::string& output, const __u8 (&in)[S]) {
+	const char* p = reinterpret_cast<const char*>(in);
+	output.assign(p, strnlen(p, S));
+}
+
+bool getGrabberDevice(const char* dev, GrabberDevice& d) {
+	int fd;
+	struct v4l2_capability video_cap;
+
+	if((fd = open(dev, O_RDONLY)) == -1) {
+		Logger::get().logWarning("Could not open device %s", dev);
+		return false;
+	}
+
+	int vcrv = ioctl(fd, VIDIOC_QUERYCAP, &video_cap);
+	close(fd);
+	if (vcrv == -1) {
+		Logger::get().logWarning("Could not read from device %s", dev);
+		return false;
+	}
+	if (!(video_cap.capabilities & V4L2_CAP_VIDEO_CAPTURE)) {
+		Logger::get().logDebug("Device %s is not a V4L2 capture device", dev);
+		return false;
+	}
+	d.device.assign(dev);
+	setString(d.name, video_cap.card);
+	setString(d.type, video_cap.driver);
+	Logger::get().logDebug("Got device %s: card %s, type %s", dev,
+			d.name.c_str(), d.type.c_str());
+	return true;
+}
+}
 
 const vector<GrabberDevice> Util::getGrabberDevices() {
-	// Ensures ng_init() is called once
-	static int count = 0;
-	if (count++ == 0) {
-		ng_init();
-	}
-	
 	vector<GrabberDevice> devices;
-	const struct ng_vid_driver *driver = 0;
-	void *handle = 0;
-	struct stat st;
-	int fh    = -1;
-	int flags = -1;
-
-	for (int i = 0; ng_dev.video_scan[i] != 0; ++i) {
-		if (lstat(ng_dev.video_scan[i], &st) == -1) {
-			if (errno == ENOENT) {
-				continue;
-			}
-#ifndef NO_DEBUG
-			fprintf(stderr,"%s: %s\n",ng_dev.video_scan[i],strerror(errno));
-#endif
-			continue;
+	glob_t matches;
+	int globRv = glob("/dev/video*", 0, 0, &matches);
+	if (0 < matches.gl_pathc) {
+		for (char** match = matches.gl_pathv; *match; ++match) {
+			GrabberDevice gd;
+			if (getGrabberDevice(*match, gd))
+				devices.push_back(gd);
 		}
-
-		fh = open(ng_dev.video_scan[i], O_RDWR);
-		if (fh == -1) {
-			if (ENODEV == errno) {
-				continue;
-			}
-#ifndef NO_DEBUG
-			fprintf(stderr,"%s: %s\n",ng_dev.video_scan[i],strerror(errno));
-#endif
-			continue;
-		}
-		close(fh);
-
-		driver = ng_vid_open(ng_dev.video_scan[i], 0, 0, 0, &handle);
-		if (driver == 0) {
-#ifndef NO_DEBUG
-			fprintf(stderr,"%s: initialization failed\n",ng_dev.video_scan[i]);
-#endif
-			continue;
-		}
-
-		flags = driver->capabilities(handle);
-		if (flags & CAN_CAPTURE) {
-			GrabberDevice d;
-			d.device = ng_dev.video_scan[i];
-			if (driver->get_devname) {
-				d.name = driver->get_devname(handle);
-			}
-			d.type = driver->name;
-			devices.push_back(d);
-		}
-
-		driver->close(handle);
 	}
-
+	globfree(&matches);
 	vector<GrabberDevice>(devices).swap(devices);
+	if (globRv == GLOB_NOSPACE)
+		throw std::bad_alloc();
 	return devices;
 }
 
@@ -298,3 +298,6 @@ void Util::ensurePathExists(const char* path) {
 		throw DirectoryCreationException(path);
 	}
 }
+
+LocalizedError::~LocalizedError() throw() {
+}
